module.exports = {};

const kebab = (s)=>s.replace(/[a-z](?=[A-Z])|[A-Z]+(?=[A-Z][a-z])/g, '$&-').toLowerCase()
;
const toCssText = (style)=>{
    let css = '';
    for(const key in style)css += kebab(key) + ':' + style[key] + ';';
    return css;
};
const createProp = (doc = html, el, _type, name, value, attrs)=>{
    switch(name){
        case 'children':
        case 'ref':
            return;
        case 'style':
            value = value?.valueOf();
            if (typeof value === 'object') value = toCssText(value);
            el.setAttribute('style', value);
            attrs.style = el.getAttributeNode('style');
            return;
    }
    const attr = name
    ;
    value = value?.valueOf();
    switch(typeof value){
        case 'string':
        case 'number':
            if (doc === svg || !(name in el)) {
                el.setAttribute(attr, value);
                attrs[attr] = el.getAttributeNode(attr);
                return;
            }
            break;
        case 'function':
            el.setAttribute(attr, '');
            attrs[attr] = el.getAttributeNode(attr)
            ;
            el[name] = value;
            return;
    }
    el[name] = value;
    if (el.hasAttribute(attr)) attrs[attr] = el.getAttributeNode(attr);
};
const propCache = new WeakMap();
const createProps = (doc, el, type, props = {}, attrs = {}, cacheRef = el)=>{
    for(const name in props)createProp(doc, el, type, name, props[name], attrs);
    propCache.set(cacheRef, {
        props,
        attrs
    });
};
const updateProps = (doc, el, type, next = {}, cacheRef = el)=>{
    if (!propCache.has(cacheRef)) return next && createProps(doc, el, type, next, void 0, cacheRef);
    const c = propCache.get(cacheRef);
    const { attrs , props  } = c;
    if (!next) {
        for(const name in attrs)el.removeAttributeNode(attrs[name]);
        for(const name3 in props)delete el[name3];
        propCache.delete(cacheRef);
        return;
    }
    let value;
    out: for(const name in props){
        if (!(name in next)) {
            delete el[name];
            continue;
        }
        value = next[name];
        switch(name){
            case 'children':
            case 'ref':
                continue out;
            case 'value':
                value = value?.valueOf();
                el[name] !== value && document.activeElement !== el && (el[name] = value);
                continue out;
        }
        value = value?.valueOf()
        ;
        if (props[name] !== value) {
            if (typeof value === 'function') {
                const attr = name
                ;
                props[attr] = el[attr] = value;
            } else if (!(name in attrs)) {
                el[name] = value;
            }
        }
    }
    for(const name1 in attrs){
        if (!(name1 in next) || next[name1] === false || next[name1] == null) {
            el.removeAttributeNode(attrs[name1]);
            delete attrs[name1];
            continue;
        }
        value = next[name1]?.valueOf();
        switch(name1){
            case 'style':
                if (typeof value === 'object') value = toCssText(value);
                break;
        }
        if (props[name1] !== value && typeof value !== 'function') attrs[name1].value = value;
    }
    for(const name2 in next){
        if (!(name2 in attrs) && !(name2 in props)) createProp(doc, el, type, name2, next[name2], attrs);
    }
    c.props = next;
};

const anchor = new Comment();
const Fragment = Symbol();
const jsx$1 = (kind, props, key)=>kind === Fragment ? props.children : {
        kind,
        props,
        key
    }
;
const jsxs = jsx$1;
let hook;
const createHook = ()=>function current(fn = current.fn) {
        const prev = hook;
        hook = current;
        hook.fn = fn;
        fn();
        hook = prev;
    }
;
let Chunk = class Chunk extends Array {
    firstChild;
    dom = [];
    get last() {
        return this.dom.at(-1);
    }
    get nextSibling() {
        return this.last?.nextSibling;
    }
    appendChild(x) {
        this.push(x);
    }
    after(x) {
        this.last?.after(x);
    }
    save() {
        this.dom = [
            ...this
        ];
    }
    remove() {
        this.dom.forEach((el)=>el.remove()
        );
        this.splice(0);
    }
    removeChild(x) {
        const i = this.indexOf(x);
        ~i && this.splice(i, 1);
    }
};
const { TEXT_NODE , COMMENT_NODE  } = document;
const html = document.createElement.bind(document);
const svg = document.createElementNS.bind(document, 'http://www.w3.org/2000/svg');
const forceArray = (x, withNull)=>Array.isArray(x) ? withNull && !x.length ? [
        null
    ] : x : x == null && !withNull ? [] : [
        x
    ]
;
const flatDom = (arr, res = [])=>{
    for (const el of arr){
        if (el.dom) res.push(...flatDom(el.dom));
        else res.push(el);
    }
    return res;
};
const prevs = new WeakMap();
const render = (n, el, doc = html, withNull = false)=>reconcile(el, forceArray(n, withNull), prevs.get(el), doc)
;
const reconcile = (parent, nk, pk, doc)=>{
    if (pk?.running) {
        console.warn('attempt to reconcile lane which is reconciling');
        return;
    }
    if (pk === nk) nk = [
        ...nk
    ];
    prevs.set(parent, nk);
    nk.running = true;
    nk.dom = Array(nk.length);
    nk.keyed = new Map();
    nk.mapped = new Map();
    if (Array.isArray(pk)) {
        const keep = new Set();
        for(let i = 0, n, el, p, pel, k, pi; i < nk.length; i++){
            n = nk[pi = i];
            k = n?.key;
            if (k != null) {
                nk.keyed.set(k, i);
                pi = pk.keyed.get(k) ?? -1;
            }
            p = pk[pi];
            pel = pk.dom[pi];
            nk.dom[i] = el = create$1(doc, n, p, pel);
            if (el === pel) keep.add(pel);
        }
        for (const pel1 of pk.dom){
            if (!keep.has(pel1)) {
                if (parent instanceof Chunk) parent.removeChild(pel1);
                pel1.remove();
                pk.mapped.get(pel1)?.hook?.onremove?.();
            }
        }
    } else {
        for(let i = 0, n, el, k; i < nk.length; i++){
            n = nk[i];
            k = n?.key;
            if (k != null) nk.keyed.set(k, i);
            nk.dom[i] = el = create$1(doc, n);
            nk.mapped.set(el, n);
        }
    }
    nk.flatDom = flatDom(nk.dom);
    if (pk?.flatDom) diff(parent, nk.flatDom, pk.flatDom);
    else nk.flatDom.forEach((el)=>parent.appendChild(el)
    );
    nk.running = false;
};
const diff = (parent, n, p, i = 0, len = n.length, el, last)=>{
    if (parent instanceof Chunk) {
        for(; i < len; i++){
            el = n[i];
            if (i < parent.length) {
                if (p[i] === el) continue;
                parent[i] = el;
            } else {
                parent.push(el);
            }
        }
        let d = parent.length - len;
        while(d--)parent.pop();
    } else {
        for(; i < len; i++){
            el = n[i];
            if (p[i] === el) last = el;
            else if (!i) parent.insertBefore(last = el, parent.firstChild);
            else last.after(last = el);
        }
    }
};
const create$1 = (doc, n, p, pel)=>{
    let el;
    switch(typeof n){
        case 'string':
        case 'number':
            if (pel?.nodeType === TEXT_NODE) {
                if (p != n) pel.nodeValue = n;
                return pel;
            }
            el = new Text(n);
            return el;
        case 'object':
            if (!n) break;
            if (Array.isArray(n)) {
                if (pel && Array.isArray(p)) el = pel;
                else el = new Chunk();
                reconcile(el, n, p, doc);
                el.save();
            } else if (typeof n.kind === 'string') {
                if (n.kind === 'svg') doc = svg;
                if (n.props.ref?.current && n.props.ref.current.tagName.toLowerCase() === n.kind && (el = n.props.ref.current) ||
                pel && p?.kind === n.kind && (el = pel)) {
                    updateProps(doc, el, n.kind, n.props);
                } else {
                    el = doc(n.kind, 'is' in n.props ? {
                        is: n.props.is
                    } : void 0);
                    createProps(doc, el, n.kind, n.props);
                }
                if (n.kind === 'foreignObject') doc = html;
                render(n.props.children, el, doc);
                if ('ref' in n.props && el !== n.props.ref.current) queueMicrotask(()=>n.props.ref.current = el
                );
            } else {
                let initial = true;
                if (!((el = pel) && (n.hook = p.hook))) {
                    el = new Chunk();
                    n.hook = createHook();
                }
                let prevDom;
                let nextDom;
                n.hook(()=>{
                    let next;
                    if (!initial && !(next = el.nextSibling)) el.after(next = anchor);
                    render(n.kind(n.props), el, doc, true);
                    el.save();
                    if (!initial && next) {
                        nextDom = flatDom(el);
                        if (prevDom?.length > 0) {
                            for(let i = 0, e; i < nextDom.length; i++){
                                e = nextDom[i];
                                if (prevDom[i] !== e) next.before(e);
                            }
                        } else {
                            for (const e of nextDom)next.before(e);
                        }
                        prevDom = nextDom;
                        next === anchor && next.remove();
                    } else {
                        initial = false;
                    }
                });
            }
            return el;
    }
    if (pel?.nodeType === COMMENT_NODE) return pel;
    el = new Comment();
    return el;
};

const FluentFlags = (cb, flags = {})=>new Proxy(cb, {
        get (_, prop, receiver) {
            flags[prop] = true;
            return receiver;
        },
        apply (_, self, args) {
            const f = {
                ...flags
            };
            flags = {}
            ;
            return cb.call(self, f).apply(self, args);
        }
    })
;

function onEvent(el, type, listener, options) {
    el.addEventListener(type, listener, options);
    return ()=>off(el, type, listener, options)
    ;
}
 const on = ()=>FluentFlags((flags)=>(el, type, fn, options = {})=>onEvent(el, type, fn, {
                ...flags.active ? {
                    passive: false
                } : null,
                ...flags,
                ...options
            })
    )
;
 const off = (el, type, listener, options)=>el.removeEventListener(type, listener, options)
;

const accessors = (target, source, fn)=>Object.defineProperties(target, Object.fromEntries(Object.entries(source).map(([key, value])=>{
        const next = fn(key, value);
        const prev = Object.getOwnPropertyDescriptor(target, key);
        if (prev && prev.get && prev.set) {
            const { get , set  } = next;
            next.get = ()=>prev.get?.() ?? get()
            ;
            next.set = (v)=>{
                prev.set?.(v);
                set(prev.get?.() ?? v);
            };
        }
        return [
            key,
            {
                configurable: true,
                enumerable: true,
                ...next
            }
        ];
    })))
;

const Types = new Map([
    [
        String,
        (x)=>x.toString()
    ],
    [
        Number,
        (x)=>parseFloat(x)
    ],
    [
        Boolean,
        (x)=>x = x === false ? false : x != null
    ],
]);
const applyProps = (self, data)=>{
    for (const [key1, value1] of Object.entries(data))if ([
        String,
        Number,
        Boolean
    ].includes(value1)) data[key1] = void 0;
    accessors(self, data, (key)=>{
        const attr = key.toLowerCase();
        return {
            get: ()=>data[key]
            ,
            set (value) {
                self.setAttribute(attr, value);
            }
        };
    });
    return data;
};
const attrs = (attrs1)=>(superclass)=>{
        const types = new Map();
        const map = new Map();
        for (const [key2, value] of Object.entries(new attrs1())){
            const type = value == null ? Types.get(String) : Types.get(value) ?? Types.get(value?.constructor);
            if (!type) {
                throw new TypeError(`Attribute "${key2}" is not valid type, must be either: String, Number, Boolean, null, undefined`);
            }
            types.set(key2, type);
            map.set(key2.toLowerCase(), key2);
        }
        const observedAttributes = [
            ...map.keys()
        ];
        return class extends superclass {
            static get observedAttributes() {
                return observedAttributes.concat(super.observedAttributes ?? []);
            }
            #data;
            constructor(...args){
                super(...args);
                this.#data = applyProps(this, new attrs1());
            }
            attributeChangedCallback(name, oldValue, newValue) {
                let key;
                if (key = map.get(name)) {
                    const prev = this.#data[key];
                    const next = types.get(key)(newValue);
                    if (!Object.is(prev, next)) {
                        this.#data[key] = next;
                        this.propertyChangedCallback?.(key, prev, next);
                    }
                }
                super.attributeChangedCallback?.(name, oldValue, newValue);
            }
        };
    }
;

const Listener = (body)=>new Function('event', `with(this){let fn=${body};return typeof fn=='function'?fn.call(this,event):fn}`)
;
const events = ()=>(superclass)=>superclass.prototype.dispatch
         ? superclass : class extends superclass {
            dispatchEvent(event) {
                const onEvent = `on${event.type}`;
                let pass = true;
                if (!(onEvent in this.constructor.prototype)) {
                    let fn = this[onEvent];
                    if (!fn) fn = Listener(this.getAttribute(onEvent));
                    pass = fn.call(this, event);
                }
                if (pass !== false) super.dispatchEvent(event);
                return pass;
            }
            dispatch(name, detail, init) {
                return this.dispatchEvent(new CustomEvent(name, {
                    detail,
                    ...init
                }));
            }
        }
;

const lifecycle = ()=>(superclass)=>class extends mixter(superclass, events()) {
            isMounted = false;
            connectedCallback() {
                super.connectedCallback?.();
                if (!this.isMounted) {
                    this.isMounted = true;
                    this.dispatch('mounted');
                }
            }
            disconnectedCallback() {
                super.disconnectedCallback?.();
                queueMicrotask(()=>{
                    if (!this.isConnected) {
                        this.isMounted = false;
                        this.dispatch('unmounted');
                    }
                });
            }
        }
;

const props = (props1)=>(superclass)=>class extends superclass {
            constructor(...args){
                super(...args);
                Object.defineProperties(this, Object.fromEntries(Object.entries(new props1()).map(([key, value])=>[
                        key,
                        {
                            configurable: true,
                            enumerable: true,
                            value
                        },
                    ]
                )));
            }
        }
;

const shadow = (init = '', html = '')=>(superclass)=>class extends superclass {
            root = (()=>{
                const root = this.attachShadow(typeof init === 'object' ? init : {
                    mode: 'open'
                });
                root.innerHTML = typeof init === 'string' ? init : html;
                return root;
            })();
        }
;

const argsOf = (x1)=>{
    const d = x1.indexOf('{') + 1;
    if (!d || x1.slice(0, d).includes(')')) return [];
    x1 = x1.slice(d);
    let match;
    do {
        match = false;
        x1 = x1.replace(/`((\\`)?[^`])+?`|'((\\')?[^'])+?'|"((\\")?[^"])+?"|\([^{[(]*?\)|\{[^{[(]*?\}|\[[^{[(]*?\]/g, ()=>{
            match = true;
            return '';
        });
    }while (match)
    return x1.split('}')[0].split(',').map((x)=>x.split(/[^\w\s$]+/g)[0].trim()
    );
};
 const argtor = (fn, keys = new Set())=>(argsOf('' + fn).forEach((x)=>!x || keys.add(x)
    ), keys)
;

const create = (target)=>{
    const NO_DEPS = Symbol();
    const REDUCER = Symbol();
    const entries = Object.entries(target);
    const mem = Object.fromEntries(entries);
    const register = (f)=>{
        f.keys = argtor(f.fn);
        f.keys.forEach((key)=>{
            const fx = effects.get(key);
            if (!fx) {
                console.warn('No effects for key:', key);
                return;
            }
            fx.push(f);
        });
        !f.keys.size && effects.get(NO_DEPS).push(f);
        f.values = {};
        return run(f);
    };
    const update = (f)=>{
        f.pass = !f.keys.size;
        for (const key of f.keys){
            const value = target[key];
            if (value == null) {
                f.pass = false;
                return;
            }
            if (!Object.is(f.values[key], value)) {
                f.values[key] = value;
                f.pass = true;
            }
        }
    };
    const run = (f)=>{
        update(f);
        f.dispose?.();
        if (!f.pass) return;
        const result = f.fn(f.values);
        if (f.cb?.(result) === false) return;
        if (f.target != null) {
            target[f.target] = result;
            return true;
        } else {
            if (typeof result === 'function') {
                f.dispose = ()=>{
                    result();
                    f.dispose = null;
                };
            }
        }
    };
    const context = {
        $: {}
    };
    const $ = context.$;
    $.$ = $;
    accessors(context.$, target, (key)=>({
            get: ()=>mem[key]
            ,
            set (v) {
                if (v === REDUCER) {
                    v = reducer.initial;
                    reducer.target = key;
                    if (register(reducer)) return;
                    if (v == null) return;
                }
                target[key] = v;
            }
        })
    );
    accessors(target, target, (key)=>({
            get: ()=>mem[key]
            ,
            set (v) {
                if (!Object.is(mem[key], v)) {
                    mem[key] = v;
                    effects.get(key).forEach(run);
                }
            }
        })
    );
    let reducer;
    $.reduce = (fn, initial)=>{
        reducer = {
            fn,
            initial
        };
        return REDUCER;
    };
    $.effect = (fn, cb)=>register({
            fn,
            cb
        })
    ;
    let effects;
    $.cleanup = ()=>{
        if (effects) {
            for (const fns of effects.values())fns.forEach((f)=>f.dispose?.()
            );
        }
        effects = new Map([
            [
                NO_DEPS,
                []
            ],
            ...entries.map(([key])=>[
                    key,
                    []
                ]
            ),
        ]);
    };
    $.cleanup();
    return $;
};
const state = (fn)=>(superclass)=>class extends mixter(superclass, lifecycle()) {
            host = this;
            constructor(...args){
                super(...args);
                const context = create(this);
                on()(this, 'mounted', ()=>fn.call(this, context)
                );
                on()(this, 'unmounted', context.cleanup);
            }
        }
;

function mixter(a, ...mixins) {
    return mixins.reduce((c, mixin)=>mixin(c)
    , a);
}

const observeNodes = (nodes, observer, options)=>nodes?.forEach((node)=>observer.observe(node, options)
    )
;
const nodesToText = (nodes)=>nodes?.map((node)=>{
        const text = node.textContent;
        return text.trim().length ? text : '';
    }).join('') ?? ''
;
const slotted = (slots)=>({
        get nodes () {
            return slots.map((slot)=>slot.assignedNodes()
            ).flat(Infinity);
        },
        get elements () {
            return slots.map((slot)=>slot.assignedElements()
            ).flat(Infinity);
        },
        get firstChild () {
            return this.elements[0];
        }
    })
;
const onSlotChange = (el1, cb, fn = (el)=>[
        ...el.querySelectorAll('slot')
    ]
)=>on()(el1, 'slotchange', ()=>cb(slotted(fn(el1)))
    )
;
const onTextChange = (el, cb, fn)=>{
    let observer;
    const off = onSlotChange(el, ({ nodes  })=>{
        observer?.disconnect();
        if (nodes.length) {
            observer = new MutationObserver(()=>cb(nodesToText(nodes))
            );
            observeNodes(nodes, observer, {
                characterData: true
            });
            cb(nodesToText(nodes));
        }
    }, fn);
    return ()=>{
        observer?.disconnect();
        off();
    };
};

const Registry = new WeakMap();
const fromElement = (Element, options = {})=>{
    let fn = Registry.get(Element);
    if (fn) return fn;
    const tag = 'x-' + Element.name.toLowerCase() + (Math.random() * 10000000 | 0).toString(36);
    const ctor = class extends Element {
    };
    customElements.define(tag, ctor, options);
    if (options.extends) fn = (props)=>jsx$1(options.extends, Object.assign(props, {
            is: tag
        }), void 0)
    ;
    else fn = (props)=>jsx$1(tag, props, void 0)
    ;
    Registry.set(Element, fn);
    fn.toString = ()=>tag
    ;
    return fn;
};

const proxy = (target)=>new Proxy(target, {
        get: (_, key)=>({
                get current () {
                    return target[key];
                },
                set current (el){
                    target[key] = el;
                }
            })
    })
;
const refs = ($)=>({
        ref: Ref($)
    })
;
const Ref = (state)=>proxy(state)
;

const renderer = (render)=>($)=>({
            render (fn) {
                $.effect(fn, (result)=>{
                    render(result, $.root);
                    return false;
                });
            }
        })
;

const jsx = ($)=>({
        ...renderer(render)($),
        part (fn, output) {
            let update;
            const cb = (value)=>{
                output = value;
                update?.();
                return false;
            };
            const Fn = ()=>{
                if (!update) $.effect(fn, cb);
                update = hook;
                return output;
            };
            return Fn;
        }
    })
;

const {replace} = '';
const ca = /[&<>'"]/g;
const esca = {
  '&': '&amp;',
  '<': '&lt;',
  '>': '&gt;',
  "'": '&#39;',
  '"': '&quot;'
};
const pe = m => esca[m];
const escape = es => replace.call(es, ca, pe);

const compile = async (def, keys = new Set())=>{
    const R = (await (await def?.default ?? def))?.default ?? def;
    if (!R) throw new ReferenceError('Could not find definition for language');
    const map = {};
    const parts = [];
    for (let [name, value] of Object.entries(R)){
        keys.add(name);
        if (Array.isArray(value)) {
            map[name] = await compile(value[1], keys);
            value = value[0];
        }
        parts.push(`(?<${name}>${value?.source ?? value})`);
    }
    const regexp = RegExp(parts.join('|'), 'gm');
    regexp.map = map;
    regexp.keys = keys;
    return regexp;
};
 const syntax = (regexp, s)=>s.replace(RegExp('(' + regexp.source + ')|(?<catchbad>[&<>"\'])', 'gm'), (...args)=>{
        const groups = args.pop();
        const entries = Object.entries(groups).filter((e)=>e[1] != null
        );
        if (!entries.length) return '';
        const [tag, value] = entries[0];
        if (tag === 'catchbad') return escape(value);
        if (regexp.map) {
            const sub = regexp.map[tag];
            if (sub) return `<span class="${tag}-outer">${syntax(sub, value)}</span>`;
        }
        return `<span class="${tag}">${escape(value)}</span>`;
    })
;

const languages = {};
const compiled = new Map();
const style$2 =  `
:host {
  display: block;
}
* {
  color: var(--color);
}
*::selection {
  background: var(--selection) !important;
}
pre {
  margin: 0;
}
slot {
  display: none;
}`;
const keyStyle = (x)=> `
.${x} {
  color: var(--${x});
  font-style: var(--${x}-style);
  font-weight: var(--${x}-weight);
}`
;
class CodeSyntaxElement extends mixter(HTMLElement, shadow( `<style>${style$2}</style><slot></slot><pre><code></code></pre>`), attrs(class {
    codeHTML = String;
    language = String;
    theme = String;
}), props(class {
    syntax;
    code;
    codeRawText;
    highlight;
    onTextContent;
}), state(({ $ , effect , reduce  })=>{
    $.code = reduce(({ root  })=>root.querySelector('code')
    );
    setTimeout(()=>{
        if ($.highlight == null) $.highlight = escape;
    }, 100);
    effect(({ language  })=>{
        if (language in languages) $.syntax = languages[language];
    });
    effect(async ({ root , syntax: syntax$1  })=>{
        let { style: style1 , regexp  } = compiled.get(syntax$1) ?? {};
        if (!regexp || !style1) {
            regexp = await compile(syntax$1);
            style1 = new CSSStyleSheet();
            await style1.replace([
                ...regexp.keys
            ].map(keyStyle).join(''));
            compiled.set(syntax$1, {
                style: style1,
                regexp
            });
        }
        root.adoptedStyleSheets = [
            style1
        ];
        $.highlight = (s)=>syntax(regexp, s)
        ;
    });
    effect(({ root  })=>onTextChange(root, (text)=>{
            $.codeRawText = text;
        })
    );
    $.codeHTML = reduce(({ codeRawText , highlight  })=>highlight(codeRawText)
    );
    effect(({ code , codeHTML  })=>{
        code.innerHTML = codeHTML;
    });
})) {
}

const style$1 =  `
:host {
  position: relative;
  overflow: hidden;
  display: inline-flex;
  width: 100%;
  height: 100%;
}

[part='scroller'] {
  pointer-events: none;
  position: absolute;
  top: 0;
  left: 0;
}

[name='leader'] {
  box-sizing: border-box;
  position: absolute;
  z-index: 1;
  left: 0;
  top: 0;
  margin: 0;
  padding: 0;
}

[name='leader']::slotted(:first-child) {
  box-sizing: border-box;
  display: inline-flex;
  width: 100%;
  height: 100%;
  margin: 0;
  resize: none;
  overflow: scroll;
}

[name='follower'] {
  display: contents;
}`;
 class SuperImposeElement extends mixter(HTMLElement, shadow( `<style>${style$1}</style><slot name="leader"></slot><div part="scroller"><slot name="follower"></slot></div>`), events(), props(class {
    scrollLeft = 0;
    scrollTop = 0;
    scroller;
    leader;
    onScroll;
}), state(({ $ , effect , reduce  })=>{
    $.scroller = reduce(({ root  })=>root.querySelector('[part=scroller]')
    );
    effect(({ root  })=>onSlotChange(root, ({ firstChild  })=>{
            $.leader = firstChild;
        })
    );
    $.onScroll = reduce(({ host  })=>(e)=>{
            const el = e.currentTarget;
            if (el) {
                Object.assign($, {
                    scrollLeft: el.scrollLeft,
                    scrollTop: el.scrollTop
                });
                host.dispatchEvent(new Event('scroll', {
                    bubbles: true
                }));
            }
        }
    );
    effect(({ leader , onScroll  })=>on()(leader, 'scroll', onScroll)
    );
    effect(({ scroller , scrollLeft , scrollTop  })=>{
        Object.assign(scroller.style, {
            left: -scrollLeft + 'px',
            top: -scrollTop + 'px'
        });
    });
})) {
}

const startOfLine = (line)=>line.match(/[^\s]|$/m).index + 1
;
const leftmost = (lines, startLine, minLeft = 1)=>{
    let left1 = lines.reduce((p, n, i)=>{
        const left = startOfLine(n);
        return left >= minLeft && left < p.col ? {
            line: i + startLine,
            col: left
        } : left === p.col ? {
            line: i + startLine,
            col: p.col
        } : p;
    }, {
        line: Infinity,
        col: Infinity
    });
    if (!isFinite(left1.line) || !isFinite(left1.col)) left1 = {
        line: startLine + lines.length + 1,
        col: 1
    };
    return left1;
};
const insert = typeof document === 'undefined' || typeof document.execCommand === 'undefined' ? (text, t = document.activeElement)=>t.value = t.value.slice(0, t.selectionStart) + text + t.value.slice(t.selectionEnd)
 : (text)=>document.execCommand('insertText', false, text)
;

const defaults = {
    tabStyle: 'spaces',
    tabSize: 2,
    comments: [
        '//',
        '/*',
        '*/'
    ]
};
class Buffer {
    #text;
    insert;
    options;
    constructor(textarea, insert, options = {}){
        this.#text = textarea;
        this.insert = insert;
        this.options = Object.assign({}, defaults, options);
    }
    get value() {
        return this.#text.value;
    }
    get tab() {
        return this.options.tabStyle === 'tabs' ? '\t' : ' '.repeat(this.options.tabSize);
    }
    get numberOfLines() {
        return ~this.value.indexOf('\n') && this.value.split('\n').length || 1;
    }
    static lineAt(value, line) {
        return value.split(/\n/).at(line);
    }
    lineAt(line) {
        return Buffer.lineAt(this.value, line);
    }
    scrollIntoView(pos) {
        const { selectionStart , selectionEnd , selectionDirection  } = this.#text;
        if (pos == null) pos = selectionDirection === 'forward' ? selectionEnd : selectionStart;
        this.#text.setSelectionRange(pos, pos);
        this.#text.blur();
        this.#text.focus();
        this.#text.setSelectionRange(selectionStart, selectionEnd, selectionDirection);
    }
    getRange() {
        const { selectionStart , selectionEnd , selectionDirection  } = this.#text;
        const [start, end] = [
            selectionStart,
            selectionEnd
        ].map(this.getLineCol);
        const [head, tail] = selectionDirection === 'forward' ? [
            end,
            start
        ] : [
            start,
            end
        ];
        return {
            start,
            end,
            head,
            tail,
            hasSelection: selectionStart !== selectionEnd,
            selectionStart,
            selectionEnd,
            selectionDirection
        };
    }
    getLineCol = (n)=>Buffer.getLineCol(this.value, n)
    ;
    static getLineCol(value, n) {
        let line = 1;
        let col = 1;
        for(let i = 0; i < n; i++){
            if (value.charAt(i) === '\n') {
                line++;
                col = 0;
            }
            col++;
        }
        return {
            line,
            col
        };
    }
    getPositionFromLineCol(pos) {
        return Buffer.getPositionFromLineCol(this.value, pos);
    }
    static getPositionFromLineCol(value, point) {
        const { line , col  } = point;
        const lines = value.split('\n');
        if (line > lines.length) return value.length;
        if (line < 1) return 0;
        const chunk = lines.slice(0, line);
        chunk[chunk.length - 1] = chunk.at(-1).slice(0, Math.max(1, col) - 1);
        return chunk.join('\n').length;
    }
    getArea({ start , end  }) {
        const startPos = this.getPositionFromLineCol(start);
        const endPos = this.getPositionFromLineCol(end);
        return [
            startPos,
            endPos
        ];
    }
    setSelectionRange(start, end, direction) {
        start = Math.max(0, start);
        end = Math.min(this.value.length, end);
        this.#text.setSelectionRange(start, end, direction);
    }
    moveCaretTo({ line , col  }, selection, direction = this.#text.selectionDirection) {
        const headPos = this.getPositionFromLineCol({
            line,
            col
        });
        const tailPos = selection ? this.getPositionFromLineCol(selection) : headPos;
        if (selection) {
            const pos = Math[direction === 'backward' ? 'min' : 'max'](headPos, tailPos);
            this.setSelectionRange(pos, pos);
            this.scrollIntoView();
        }
        this.setSelectionRange(Math.min(headPos, tailPos), Math.max(headPos, tailPos), headPos < tailPos ? 'backward' : 'forward');
        this.scrollIntoView();
    }
    replaceBlock(replacer) {
        const { start , end , hasSelection , selectionDirection  } = this.getRange();
        const first = this.lineAt(start.line - 1);
        const last = this.lineAt(end.line - 1);
        const notch = end.col > 1 || !hasSelection ? 1 : 0;
        const sliceStart = this.getPositionFromLineCol({
            line: start.line,
            col: 1
        });
        const sliceEnd = this.getPositionFromLineCol({
            line: end.line + notch,
            col: 1
        });
        const slice = this.value.slice(sliceStart, sliceEnd);
        const { diff , text , left  } = replacer(slice, start.line);
        if (text.length === sliceEnd - sliceStart) return;
        this.setSelectionRange(sliceStart, sliceEnd);
        this.insert(text);
        this.scrollIntoView(this.getPositionFromLineCol({
            line: end.line - +!notch,
            col: left.col + diff
        }));
        const stillCaret = !hasSelection && start.col <= left.col + diff && end.col <= left.col + diff && first.length !== 0;
        const startCol = this.lineAt(start.line - 1) === first ? start.col : stillCaret ? start.col : start.col + diff;
        const endCol = stillCaret ? end.col : this.lineAt(end.line - 1) === last ? end.col : slice.length === 0 ? end.col + diff : Math.max(1 + (start.line !== end.line ? notch : 0), end.col + notch * diff);
        this.setSelectionRange(this.getPositionFromLineCol({
            line: start.line,
            col: startCol
        }), this.getPositionFromLineCol({
            line: end.line,
            col: endCol
        }), selectionDirection);
    }
    toggleSingleComment() {
        const comment = this.options.comments[0];
        this.replaceBlock((text, startLine)=>{
            const left = leftmost(text.split('\n').filter((x)=>x.length
            ), startLine);
            let diff;
            if (text.trimStart().slice(0, comment.length) === comment) {
                diff = -3;
                text = text.replace(new RegExp(`^([^${comment[0]}]*)${comment} ?`, 'gm'), '$1');
            } else {
                diff = +3;
                text = text.length === 0 ? comment + ' ' : text.replace(new RegExp(`^(?!$)([^${comment[0]}]{0,${left.col - 1}})`, 'gm'), `$1${comment} `);
            }
            return {
                diff,
                text,
                left
            };
        });
    }
    toggleDoubleComment() {
        let { selectionStart , selectionEnd , selectionDirection  } = this.getRange();
        let slice = this.value.slice(selectionStart, selectionEnd);
        const c = this.options.comments;
        const expanded = this.value.slice(selectionStart - c[1].length, selectionEnd + c[2].length);
        if (expanded.indexOf(c[1]) === 0 && expanded.lastIndexOf(c[2]) === expanded.length - c[2].length) {
            slice = expanded;
            selectionStart -= c[1].length;
            selectionEnd += c[2].length;
            this.setSelectionRange(selectionStart, selectionEnd, selectionDirection);
        }
        if (slice.indexOf(c[1]) === 0 && slice.lastIndexOf(c[2]) === slice.length - c[2].length) {
            const ins = slice.slice(c[1].length, -c[2].length);
            this.insert(ins);
            this.setSelectionRange(selectionStart, selectionEnd - c[1].length - c[2].length, selectionDirection);
        } else {
            this.insert(c[1] + slice + c[2]);
            const length = c[1].length;
            this.setSelectionRange(selectionStart + length, selectionEnd + length, selectionDirection);
        }
        this.scrollIntoView();
    }
    indent(unindent) {
        this.replaceBlock((text, startLine)=>{
            const left = leftmost(text.split('\n'), startLine, 2);
            const tabSize = this.options.tabStyle === 'tabs' ? 1 : this.options.tabSize;
            let diff;
            if (unindent) {
                diff = -tabSize;
                text = text.replace(new RegExp(`^(\t| {1,${tabSize}})`, 'gm'), '');
            } else {
                diff = +tabSize;
                text = text.length === 0 ? this.tab : text.replace(/^[^\n]/gm, `${this.tab}$&`);
            }
            return {
                diff,
                text,
                left
            };
        });
    }
    moveCaretHome(withSelection) {
        const { head , tail  } = this.getRange();
        let begin = startOfLine(this.lineAt(head.line - 1));
        if (head.col === begin) begin = 1;
        this.moveCaretTo({
            line: head.line,
            col: begin
        }, withSelection ? tail : null);
    }
    moveCaretEnd(withSelection) {
        const { head , tail  } = this.getRange();
        this.moveCaretTo({
            line: head.line,
            col: this.lineAt(head.line - 1).length + 1
        }, withSelection ? tail : null);
    }
    moveCaretLines(lines, withSelection) {
        const { head , tail  } = this.getRange();
        const { numberOfLines  } = this;
        let target = head.line + lines;
        if (target < 1 && head.line > 1) target = 1;
        else if (target > numberOfLines && head.line < numberOfLines) target = numberOfLines;
        this.moveCaretTo({
            line: target,
            col: head.col
        }, withSelection ? tail : null);
    }
    moveLines(diff) {
        const { start , end , hasSelection , selectionStart , selectionEnd , selectionDirection  } = this.getRange();
        const lines = this.value.split('\n');
        const notch = end.col === 1 && hasSelection ? 1 : 0;
        const down = diff > 0;
        let startLine = start.line + (!down ? diff : 0);
        let endLine = end.line + (down ? diff : 0) - notch;
        if (down) {
            const adj = endLine - lines.length;
            if (adj > 0) {
                endLine -= adj;
                diff -= adj;
            }
        } else {
            const adj = 1 - startLine;
            if (adj > 0) {
                startLine += adj;
                diff += adj;
            }
        }
        if (!diff) return;
        const area = {
            start: {
                line: startLine,
                col: 1
            },
            end: {
                line: endLine,
                col: this.lineAt(endLine - 1).length + 1
            }
        };
        const [sliceStart, sliceEnd] = this.getArea(area);
        const slice = this.value.slice(sliceStart, sliceEnd);
        const part = slice.split('\n');
        const cut = down ? end.line - notch - start.line + 1 : -diff;
        const top = part.slice(0, cut);
        const bottom = part.slice(cut);
        this.setSelectionRange(sliceStart, sliceEnd, selectionDirection);
        this.insert(bottom.concat(top).join('\n'));
        const diffPos = down ? bottom.join('\n').length + 1 : -(top.join('\n').length + 1);
        const selStart = selectionStart + diffPos;
        const selEnd = selectionEnd + diffPos;
        this.setSelectionRange(selStart, selEnd, selectionDirection);
        this.scrollIntoView(down ? selEnd : selStart);
    }
    duplicate() {
        const { start , end , hasSelection , selectionStart , selectionEnd , selectionDirection  } = this.getRange();
        const { numberOfLines  } = this;
        if (!hasSelection) {
            start.col = 1;
            end.col = 1;
            end.line++;
        }
        const [sliceStart, sliceEnd] = this.getArea({
            start,
            end
        });
        let slice = this.value.slice(sliceStart, sliceEnd);
        if (!hasSelection && slice.at(-1) !== '\n') {
            if (numberOfLines > 1) slice = this.value.slice(sliceStart - 1, sliceEnd);
            else slice = '\n' + slice;
        }
        this.setSelectionRange(sliceEnd, sliceEnd);
        this.insert(slice);
        const length = slice.length;
        if (!hasSelection) this.setSelectionRange(selectionStart + length, selectionEnd + length);
        else this.setSelectionRange(sliceEnd, sliceEnd + length, selectionDirection);
        this.scrollIntoView();
    }
    deleteLine() {
        const { start , end , hasSelection  } = this.getRange();
        const { numberOfLines  } = this;
        const before = {
            line: start.line,
            col: start.col
        };
        if (!hasSelection) {
            if (end.line < numberOfLines) {
                start.col = 1;
                end.col = 1;
                end.line++;
            } else if (numberOfLines === 1) {
                start.col = 1;
                end.col = this.value.length + 1;
            } else {
                before.line--;
                start.line--;
                start.col = this.lineAt(end.line - 2).length + 1;
                end.col = this.lineAt(end.line - 1).length + 1;
            }
        }
        const [sliceStart, sliceEnd] = this.getArea({
            start,
            end
        });
        this.setSelectionRange(sliceStart, sliceEnd);
        this.insert('');
        const pos = this.getPositionFromLineCol(before);
        this.setSelectionRange(pos, pos);
        this.scrollIntoView();
    }
}

class TextAreaCodeElement extends mixter(HTMLTextAreaElement, attrs(class {
    tabSize = 2;
    tabStyle = 'spaces';
    comments = '// /* */';
}), props(class {
    buffer;
    pageSize;
    viewHeight;
    lineHeight = 16;
    onKeyDown;
}), state(({ $ , effect , reduce  })=>{
    effect(({ host  })=>{
        host.style.whiteSpace = 'pre';
        host.setAttribute('wrap', 'off');
        host.setAttribute('spellcheck', 'false');
        host.setAttribute('autocorrect', 'off');
        host.setAttribute('autocomplete', 'off');
    });
    $.buffer = reduce(({ host  })=>new Buffer(host, insert)
    );
    effect(({ buffer , comments , tabSize , tabStyle  })=>{
        Object.assign(buffer.options, {
            tabSize,
            tabStyle,
            comments: comments.split(' ')
        });
    });
    $.pageSize = reduce(({ viewHeight , lineHeight  })=>Math.floor(viewHeight / lineHeight) - 1
    );
    effect(({ host , lineHeight  })=>{
        const observer = new ResizeObserver((entries)=>{
            $.viewHeight = entries[0].contentBoxSize[0].blockSize;
            $.lineHeight = parseFloat(window.getComputedStyle(host).getPropertyValue('line-height')) || lineHeight;
        });
        observer.observe(host);
        return ()=>observer.disconnect()
        ;
    });
    $.onKeyDown = reduce(({ host , buffer , pageSize  })=>(e)=>{
            const b = buffer;
            const cmdKey = e.ctrlKey || e.metaKey;
            if (cmdKey) {
                if ('/' === e.key) {
                    e.preventDefault();
                    b.toggleSingleComment();
                    return;
                }
                if ('?' === e.key) {
                    e.preventDefault();
                    b.toggleDoubleComment();
                    return;
                }
                if ('D' === e.key) {
                    e.preventDefault();
                    b.duplicate();
                    return;
                }
            }
            if (e.altKey || cmdKey && e.shiftKey) {
                if ([
                    'ArrowUp',
                    'ArrowDown',
                    'PageUp',
                    'PageDown'
                ].includes(e.key)) {
                    e.preventDefault();
                    b.moveLines({
                        ArrowUp: -1,
                        ArrowDown: 1,
                        PageUp: -pageSize,
                        PageDown: +pageSize
                    }[e.key]);
                    return;
                }
            }
            if (e.shiftKey && e.key === 'Delete') {
                e.preventDefault();
                b.deleteLine();
                return;
            }
            if (!cmdKey && !e.altKey) {
                if ('Tab' === e.key) {
                    e.preventDefault();
                    const { selectionStart , selectionEnd  } = host;
                    const hasSelection = selectionStart !== selectionEnd;
                    if (hasSelection || e.shiftKey) b.indent(e.shiftKey);
                    else {
                        b.insert(b.tab);
                        b.scrollIntoView();
                    }
                    return;
                }
                if ('Home' === e.key) {
                    e.preventDefault();
                    b.moveCaretHome(e.shiftKey);
                    return;
                }
                if ('End' === e.key) {
                    e.preventDefault();
                    b.moveCaretEnd(e.shiftKey);
                    return;
                }
                if ([
                    'PageUp',
                    'PageDown'
                ].includes(e.key)) {
                    e.preventDefault();
                    b.moveCaretLines(e.key === 'PageUp' ? -pageSize : +pageSize, e.shiftKey);
                    return;
                }
            }
            if (!cmdKey && !e.altKey && !e.shiftKey) {
                if ('Enter' === e.key) {
                    const { start , selectionStart  } = b.getRange();
                    const line = b.lineAt(start.line - 1);
                    const indent = startOfLine(line);
                    if (indent > 0) {
                        e.preventDefault();
                        let ins = '\n' + line.slice(0, indent - 1);
                        const open = '{[(';
                        const match = open.indexOf(line.at(-1));
                        if (~match && start.col === line.length + 1) ins += b.tab;
                        const pos = selectionStart + ins.length;
                        b.insert(ins);
                        b.setSelectionRange(pos, pos);
                        b.scrollIntoView();
                        return;
                    }
                }
            }
        }
    );
    effect(({ host , onKeyDown  })=>on()(host, 'keydown', onKeyDown)
    );
})) {
}

const style = /*css*/ `
:host {
  --caret: #fff;
  --selection: #742;
  --padding: 0;
  position: relative;
  display: block;
  overflow: hidden;
  background: var(--background, transparent);
}
slot {
  display: none;
}
pre,
textarea {
  display: flex;
  white-space: pre;
  padding: 0;
  margin: 0;
  width: 100%;
  height: 100%;
  font-size: inherit;
  line-height: inherit;
  background: transparent;
  color: transparent;
  caret-color: var(--caret);
  outline: none;
  resize: none;
  border: none;
  z-index: 1;
}
pre {
  position: absolute;
  width: auto;
  height: auto;
  z-index: -1;
}
textarea::selection {
  color: var(--color);
  background-color: var(--selection) !important;
}`;
/**
 * CodeEdit custom element
 *
 * ```js
 * import { CodeEditElement } from 'code-edit'
 * customElements.define('code-edit', CodeEditElement)
 * ```
 *
 * ```html
 * <code-edit autofocus autoresize language="js" theme="deep"> ... </code-edit>
 * ```
 */
class CodeEditElement extends mixter(HTMLElement, shadow(), events(), attrs(class {
    /** The editor's value */
    value = '';
    /** Autofocus to the element when page loads */
    autoFocus = false;
    /** Whether to autoresize to the height of the contents */
    autoResize = false;
    /** Language for syntax highlighting */
    language = '';
    /** Theme to use */
    theme = '';
    /** Tab size */
    tabSize = 2;
    /** Tab style */
    tabStyle = 'spaces';
    /** Comment */
    comments = '// /* */';
}), props(class {
    /** @private */
    CodeSyntax = fromElement(CodeSyntaxElement);
    /** @private */
    SuperImpose = fromElement(SuperImposeElement);
    /** @private */
    TextAreaCode = fromElement(TextAreaCodeElement, { extends: 'textarea' });
    /** Passing a syntax definition regexp manually */
    syntax = {};
    /** @private */
    textarea;
    /** @private */
    layout;
    /** @private */
    height;
    /** @private */
    onInput;
    /** @private */
    onScroll;
}), state(({ $, effect, reduce }) => {
    const { part, render } = jsx($);
    const { ref } = refs($);
    $.onInput = reduce(({ textarea }) => (() => {
        $.value = textarea.value;
    }), () => { });
    $.onScroll = reduce(({ host }) => (() => {
        host.dispatchEvent(new Event('scroll', { bubbles: true }));
    }), () => { });
    effect(({ autoResize, layout, height, value: _ }) => {
        if (autoResize) {
            // push to the end of the job when height has actually changed
            queueMicrotask(() => {
                layout.style.height = (height.getBoundingClientRect().height ?? 'auto') + 'px';
            });
        }
    });
    effect(({ autoFocus, textarea }) => {
        if (autoFocus) {
            // deal with browser autofocus warning on initial load
            setTimeout(() => {
                textarea.focus();
                // move the caret to the beginning
                setTimeout(() => textarea.setSelectionRange(0, 0));
            });
        }
    });
    effect(({ root }) => onTextChange(root, text => {
        $.value = text;
    }));
    const Content = part(({ SuperImpose, TextAreaCode, CodeSyntax, onScroll, onInput, comments, tabSize, tabStyle, theme, language, syntax, value, }) => (jsxs(Fragment, { children: [jsxs("pre", { ref: ref.height, children: [value, ' '] }), jsxs(SuperImpose, { part: "layout", ref: ref.layout, onscroll: onScroll, children: [jsx$1(TextAreaCode, { part: "textarea", slot: "leader", ref: ref.textarea, comments: comments, oninput: onInput, tabSize: tabSize, tabStyle: tabStyle, theme: theme, value: value }), jsx$1(CodeSyntax, { part: "syntax", slot: "follower", language: language, syntax: syntax, theme: theme, children: value })] })] })));
    render(() => (jsxs(Fragment, { children: [jsx$1("style", { children: style }), jsx$1("slot", {}), jsx$1(Content, {})] })));
})) {
    get scrollLeft() {
        return this.textarea?.scrollLeft ?? 0;
    }
    get scrollTop() {
        return this.textarea?.scrollTop ?? 0;
    }
}

export { CodeEditElement, languages };
//# sourceMappingURL=code-edit.js.map
